@@ -29,6 +29,7 @@ angular.module('syncthing.core')
         $scope.myID = '';
         $scope.devices = [];
         $scope.deviceRejections = {};
+        $scope.discoveryCache = {};
         $scope.folderRejections = {};
         $scope.protocolChanged = false;
         $scope.reportData = {};
@@ -49,6 +50,7 @@ angular.module('syncthing.core')
         $scope.failedCurrentFolder = undefined;
         $scope.failedPageSize = 10;
         $scope.scanProgress = {};
+        $scope.themes = [];
 
         $scope.localStateTotal = {
             bytes: 0,
@@ -84,10 +86,12 @@ angular.module('syncthing.core')
             console.log('UIOnline');
 
             refreshSystem();
+            refreshDiscoveryCache();
             refreshConfig();
             refreshConnectionStats();
             refreshDeviceStats();
             refreshFolderStats();
+            refreshThemes();
 
             $http.get(urlbase + '/system/version').success(function (data) {
                 if ($scope.version.version && $scope.version.version !== data.version) {
@@ -171,6 +175,12 @@ angular.module('syncthing.core')
                 if (data.to === 'scanning') {
                     delete $scope.scanProgress[data.folder];
                 }
+
+                // If a folder finished scanning, then refresh folder stats
+                // to update last scan time.
+                if(data.from === 'scanning' && data.to === 'idle') {
+                    refreshFolderStats();
+                }
             }
         });
 
@@ -410,6 +420,22 @@ angular.module('syncthing.core')
             }).error($scope.emitHTTPError);
         }
 
+        function refreshDiscoveryCache() {
+            $http.get(urlbase + '/system/discovery').success(function (data) {
+                for (var device in data) {
+                    for (var i = 0; i < data[device].addresses.length; i++) {
+                        // Relay addresses are URLs with
+                        // .../?foo=barlongstuff that we strip away here. We
+                        // remove the final slash as well for symmetry with
+                        // tcp://192.0.2.42:1234 type addresses.
+                        data[device].addresses[i] = data[device].addresses[i].replace(/\/\?.*/, '');
+                    }
+                }
+                $scope.discoveryCache = data;
+                console.log("refreshDiscoveryCache", data);
+            }).error($scope.emitHTTPError);
+        }
+
         function recalcLocalStateTotal () {
             $scope.localStateTotal = {
                 bytes: 0,
@@ -585,13 +611,23 @@ angular.module('syncthing.core')
                     if ($scope.folderStats[folder].lastFile) {
                         $scope.folderStats[folder].lastFile.at = new Date($scope.folderStats[folder].lastFile.at);
                     }
+
+                    $scope.folderStats[folder].lastScan = new Date($scope.folderStats[folder].lastScan);
+                    $scope.folderStats[folder].lastScanDays = (new Date() - $scope.folderStats[folder].lastScan) / 1000 / 86400;
                 }
                 console.log("refreshfolderStats", data);
             }).error($scope.emitHTTPError);
         }, 2500);
 
+        var refreshThemes = debounce(function () {
+            $http.get("themes.json").success(function (data) { // no urlbase here as this is served by the asset handler
+                $scope.themes = data.themes;
+            }).error($scope.emitHTTPError);
+        }, 2500);
+
         $scope.refresh = function () {
             refreshSystem();
+            refreshDiscoveryCache();
             refreshConnectionStats();
             refreshErrors();
         };
@@ -606,10 +642,6 @@ angular.module('syncthing.core')
                 return 'unknown';
             }
 
-            if (folderCfg.devices.length <= 1) {
-                return 'unshared';
-            }
-
             if ($scope.model[folderCfg.id].invalid) {
                 return 'stopped';
             }
@@ -621,6 +653,13 @@ angular.module('syncthing.core')
             if (state === 'idle' && $scope.model[folderCfg.id].needFiles > 0) {
                 return 'outofsync';
             }
+            if (state === 'scanning') {
+                return state;
+            }
+
+            if (folderCfg.devices.length <= 1) {
+                return 'unshared';
+            }
 
             return state;
         };
@@ -637,12 +676,12 @@ angular.module('syncthing.core')
             if (status === 'unknown') {
                 return 'info';
             }
-            if (status === 'unshared') {
-                return 'warning';
-            }
             if (status === 'stopped' || status === 'outofsync' || status === 'error') {
                 return 'danger';
             }
+            if (status === 'unshared') {
+                return 'warning';
+            }
 
             return 'info';
         };
@@ -780,6 +819,70 @@ angular.module('syncthing.core')
             return 'info';
         };
 
+        $scope.syncthingStatus = function () {
+            var syncCount = 0;
+            var notifyCount = 0;
+            var pauseCount = 0;
+
+            // loop through all folders
+            var folderListCache = $scope.folderList();
+            for (var i = 0; i < folderListCache.length; i++) {
+                var status = $scope.folderStatus(folderListCache[i]);
+                switch (status) {
+                    case 'syncing':
+                        syncCount++;
+                        break;
+                    case 'stopped':
+                    case 'unknown':
+                    case 'outofsync':
+                    case 'error':
+                        notifyCount++;
+                        break;
+                }
+            }
+
+            // loop through all devices
+            var deviceCount = $scope.devices.length;
+            for (var i = 0; i < $scope.devices.length; i++) {
+                var status = $scope.deviceStatus({
+                    deviceID:$scope.devices[i].deviceID
+                });
+                switch (status) {
+                    case 'unknown':
+                        notifyCount++;
+                        break;
+                    case 'paused':
+                        pauseCount++;
+                        break;
+                    case 'unused':
+                        deviceCount--;
+                        break;
+                }
+            }
+
+            // enumerate notifications
+            if ($scope.openNoAuth || !$scope.configInSync || Object.keys($scope.deviceRejections).length > 0 || Object.keys($scope.folderRejections).length > 0 || $scope.errorList().length > 0 || !online) {
+                notifyCount++;
+            }
+
+            // at least one folder is syncing
+            if (syncCount > 0) {
+                return 'sync';
+            }
+
+            // a device is unknown or a folder is stopped/unknown/outofsync/error or some other notification is open or gui offline
+            if (notifyCount > 0) {
+                return 'notify';
+            }
+
+            // all used devices are paused except (this) one
+            if (pauseCount === deviceCount-1) {
+                return 'pause';
+            }
+
+            return 'default';
+        };
+
         $scope.deviceAddr = function (deviceCfg) {
             var conn = $scope.connections[deviceCfg.deviceID];
             if (conn && conn.connected) {
@@ -944,10 +1047,6 @@ angular.module('syncthing.core')
             });
         };
 
-        $scope.upgradeMajor = function () {
-            $('#majorUpgrade').modal();
-        };
-
         $scope.shutdown = function () {
             restarting = true;
             $http.post(urlbase + '/system/shutdown').success(function () {
@@ -968,12 +1067,6 @@ angular.module('syncthing.core')
             $('#editDevice').modal();
         };
 
-        $scope.idDevice = function (deviceCfg) {
-            $scope.currentDevice = deviceCfg;
-            $('#editDevice').modal('hide');
-            $('#idqr').modal('show');
-        };
-
         $scope.addDevice = function (deviceID, name) {
             return $http.get(urlbase + '/system/discovery')
                 .success(function (registry) {
@@ -1145,6 +1238,18 @@ angular.module('syncthing.core')
             }).error($scope.emitHTTPError);
         });
 
+        $scope.loadFormIntoScope = function (form) {
+            console.log('loadFormIntoScope',form.$name);
+            switch (form.$name) {
+                case 'deviceEditor':
+                    $scope.deviceEditor = form;
+                    break;
+                case 'folderEditor':
+                    $scope.folderEditor = form;
+                    break;
+            }
+        }
+
         $scope.editFolder = function (folderCfg) {
             $scope.currentFolder = angular.copy(folderCfg);
             if ($scope.currentFolder.path.slice(-1) === $scope.system.pathSeparator) {
@@ -1342,8 +1447,7 @@ angular.module('syncthing.core')
                 var devices = $scope.folders[folderID].devices;
                 for (var i = 0; i < devices.length; i++) {
                     if (devices[i].deviceID === deviceCfg.deviceID) {
-                        var label = $scope.folders[folderID].label;
-                        folders.push(label.length > 0 ? label : folderID);
+                        folders.push(folderID);
                         break;
                     }
                 }
@@ -1353,6 +1457,11 @@ angular.module('syncthing.core')
             return folders;
         };
 
+        $scope.folderLabel = function (folderID) {
+            var label = $scope.folders[folderID].label;
+            return label.length > 0 ? label : folderID;
+        }
+
         $scope.deleteFolder = function (id) {
             $('#editFolder').modal('hide');
             if (!$scope.editingExisting) {
@@ -1376,20 +1485,11 @@ angular.module('syncthing.core')
             $http.get(urlbase + '/db/ignores?folder=' + encodeURIComponent($scope.currentFolder.id))
                 .success(function (data) {
                     data.ignore = data.ignore || [];
-
-                    $('#editFolder').modal('hide')
-                        .one('hidden.bs.modal', function() {
-                            var textArea = $('#editIgnores textarea');
-
-                            textArea.val(data.ignore.join('\n'));
-
-                            $('#editIgnores').modal()
-                                .one('hidden.bs.modal', function () {
-                                    $('#editFolder').modal();
-                                })
-                                .one('shown.bs.modal', function () {
-                                    textArea.focus();
-                                });
+                    var textArea = $('#editIgnores textarea');
+                    textArea.val(data.ignore.join('\n'));
+                    $('#editIgnores').modal()
+                        .one('shown.bs.modal', function () {
+                            textArea.focus();
                         });
                 })
                 .then(function () {
@@ -1413,16 +1513,6 @@ angular.module('syncthing.core')
             });
         };
 
-        $scope.showURPreview = function () {
-            $('#settings').modal('hide')
-                .one('hidden.bs.modal', function() {
-                    $('#urPreview').modal()
-                        .one('hidden.bs.modal', function () {
-                            $('#settings').modal();
-                        });
-                });
-        };
-
         $scope.acceptUR = function () {
             $scope.config.options.urAccepted = 1000; // Larger than the largest existing report version
             $scope.saveConfig();
@@ -1467,10 +1557,6 @@ angular.module('syncthing.core')
             $http.post(urlbase + "/db/override?folder=" + encodeURIComponent(folder));
         };
 
-        $scope.about = function () {
-            $('#about').modal('show');
-        };
-
         $scope.advanced = function () {
             $scope.advancedConfig = angular.copy($scope.config);
             $('#advanced').modal('show');
@@ -1554,6 +1640,14 @@ angular.module('syncthing.core')
             });
         };
 
-        // pseudo main. called on all definitions assigned
-        initController();
+        $scope.modalLoaded = function () {
+
+            // once all modal elements have been processed
+            if ($('modal').length === 0) {
+
+                // pseudo main. called on all definitions assigned
+                initController();
+            }
+        }
+
     });
