@@ -8,7 +8,6 @@ package connections
 
 import (
 	"crypto/tls"
-	"encoding/binary"
 	"errors"
 	"fmt"
 	"io"
@@ -37,6 +36,8 @@ var (
 	listeners = make(map[string]listenerFactory, 0)
 )
 
+const perDeviceWarningRate = 1.0 / (15 * 60) // Once per 15 minutes
+
 // Service listens and dials all configured unconnected devices, via supported
 // dialers. Successful connections are handed to the model.
 type Service struct {
@@ -153,12 +154,22 @@ next:
 			continue
 		}
 
-		hello, err := exchangeHello(c, s.model.GetHello(remoteID))
+		c.SetDeadline(time.Now().Add(20 * time.Second))
+		hello, err := protocol.ExchangeHello(c, s.model.GetHello(remoteID))
 		if err != nil {
-			l.Infof("Failed to exchange Hello messages with %s (%s): %s", remoteID, c.RemoteAddr(), err)
+			if protocol.IsVersionMismatch(err) {
+				// The error will be a relatively user friendly description
+				// of what's wrong with the version compatibility
+				msg := fmt.Sprintf("Connecting to %s (%s): %s", remoteID, c.RemoteAddr(), err)
+				warningFor(remoteID, msg)
+			} else {
+				// It's something else - connection reset or whatever
+				l.Infof("Failed to exchange Hello messages with %s (%s): %s", remoteID, c.RemoteAddr(), err)
+			}
 			c.Close()
 			continue
 		}
+		c.SetDeadline(time.Time{})
 
 		s.model.OnHello(remoteID, c.RemoteAddr(), hello)
 
@@ -553,54 +564,6 @@ func (s *Service) getListenerFactory(cfg config.Configuration, uri *url.URL) (li
 	return listenerFactory, nil
 }
 
-func exchangeHello(c net.Conn, h protocol.HelloMessage) (protocol.HelloMessage, error) {
-	if err := c.SetDeadline(time.Now().Add(2 * time.Second)); err != nil {
-		return protocol.HelloMessage{}, err
-	}
-	defer c.SetDeadline(time.Time{})
-
-	header := make([]byte, 8)
-	msg := h.MustMarshalXDR()
-
-	binary.BigEndian.PutUint32(header[:4], protocol.HelloMessageMagic)
-	binary.BigEndian.PutUint32(header[4:], uint32(len(msg)))
-
-	if _, err := c.Write(header); err != nil {
-		return protocol.HelloMessage{}, err
-	}
-
-	if _, err := c.Write(msg); err != nil {
-		return protocol.HelloMessage{}, err
-	}
-
-	if _, err := io.ReadFull(c, header); err != nil {
-		return protocol.HelloMessage{}, err
-	}
-
-	if binary.BigEndian.Uint32(header[:4]) != protocol.HelloMessageMagic {
-		return protocol.HelloMessage{}, fmt.Errorf("incorrect magic")
-	}
-
-	msgSize := binary.BigEndian.Uint32(header[4:])
-	if msgSize > 1024 {
-		return protocol.HelloMessage{}, fmt.Errorf("hello message too big")
-	}
-
-	buf := make([]byte, msgSize)
-
-	var hello protocol.HelloMessage
-
-	if _, err := io.ReadFull(c, buf); err != nil {
-		return protocol.HelloMessage{}, err
-	}
-
-	if err := hello.UnmarshalXDR(buf); err != nil {
-		return protocol.HelloMessage{}, err
-	}
-
-	return hello, nil
-}
-
 func filterAndFindSleepDuration(nextDial map[string]time.Time, seen []string, now time.Time) (map[string]time.Time, time.Duration) {
 	newNextDial := make(map[string]time.Time)
 
@@ -628,3 +591,19 @@ func urlsToStrings(urls []*url.URL) []string {
 	}
 	return strings
 }
+
+var warningLimiters = make(map[protocol.DeviceID]*ratelimit.Bucket)
+var warningLimitersMut = sync.NewMutex()
+
+func warningFor(dev protocol.DeviceID, msg string) {
+	warningLimitersMut.Lock()
+	defer warningLimitersMut.Unlock()
+	lim, ok := warningLimiters[dev]
+	if !ok {
+		lim = ratelimit.NewBucketWithRate(perDeviceWarningRate, 1)
+		warningLimiters[dev] = lim
+	}
+	if lim.TakeAvailable(1) == 1 {
+		l.Warnln(msg)
+	}
+}
