@@ -25,7 +25,10 @@ import (
 	"github.com/syncthing/syncthing/lib/config"
 	"github.com/syncthing/syncthing/lib/connections"
 	"github.com/syncthing/syncthing/lib/db"
+	"github.com/syncthing/syncthing/lib/ignore"
+	"github.com/syncthing/syncthing/lib/osutil"
 	"github.com/syncthing/syncthing/lib/protocol"
+	srand "github.com/syncthing/syncthing/lib/rand"
 )
 
 var device1, device2 protocol.DeviceID
@@ -142,7 +145,7 @@ func genFiles(n int) []protocol.FileInfo {
 		files[i] = protocol.FileInfo{
 			Name:     fmt.Sprintf("file%d", i),
 			Modified: t,
-			Blocks:   []protocol.BlockInfo{{0, 100, []byte("some hash bytes")}},
+			Blocks:   []protocol.BlockInfo{{Offset: 0, Size: 100, Hash: []byte("some hash bytes")}},
 		}
 	}
 
@@ -286,7 +289,7 @@ func BenchmarkRequest(b *testing.B) {
 		files[i] = protocol.FileInfo{
 			Name:     fmt.Sprintf("file%d", i),
 			Modified: t,
-			Blocks:   []protocol.BlockInfo{{0, 100, []byte("some hash bytes")}},
+			Blocks:   []protocol.BlockInfo{{Offset: 0, Size: 100, Hash: []byte("some hash bytes")}},
 		}
 	}
 
@@ -301,7 +304,7 @@ func BenchmarkRequest(b *testing.B) {
 			Priority: 10,
 		},
 		Connection: fc,
-	}, protocol.HelloMessage{})
+	}, protocol.HelloResult{})
 	m.Index(device1, "default", files, 0, nil)
 
 	b.ResetTimer()
@@ -317,7 +320,7 @@ func BenchmarkRequest(b *testing.B) {
 }
 
 func TestDeviceRename(t *testing.T) {
-	hello := protocol.HelloMessage{
+	hello := protocol.HelloResult{
 		ClientName:    "syncthing",
 		ClientVersion: "v0.9.4",
 	}
@@ -1183,7 +1186,7 @@ func genDeepFiles(n, d int) []protocol.FileInfo {
 		}
 
 		files[i].Modified = t
-		files[i].Blocks = []protocol.BlockInfo{{0, 100, []byte("some hash bytes")}}
+		files[i].Blocks = []protocol.BlockInfo{{Offset: 0, Size: 100, Hash: []byte("some hash bytes")}}
 	}
 
 	return files
@@ -1420,6 +1423,139 @@ func TestIssue3028(t *testing.T) {
 	}
 }
 
+func TestIssue3164(t *testing.T) {
+	osutil.RemoveAll("testdata/issue3164")
+	defer osutil.RemoveAll("testdata/issue3164")
+
+	if err := os.MkdirAll("testdata/issue3164/oktodelete/foobar", 0777); err != nil {
+		t.Fatal(err)
+	}
+	if err := ioutil.WriteFile("testdata/issue3164/oktodelete/foobar/file", []byte("Hello"), 0644); err != nil {
+		t.Fatal(err)
+	}
+	if err := ioutil.WriteFile("testdata/issue3164/oktodelete/file", []byte("Hello"), 0644); err != nil {
+		t.Fatal(err)
+	}
+	f := protocol.FileInfo{
+		Name: "issue3164",
+	}
+	m := ignore.New(false)
+	if err := m.Parse(bytes.NewBufferString("(?d)oktodelete"), ""); err != nil {
+		t.Fatal(err)
+	}
+
+	fl := rwFolder{
+		dbUpdates: make(chan dbUpdateJob, 1),
+		dir:       "testdata",
+	}
+
+	fl.deleteDir(f, m)
+
+	if _, err := os.Stat("testdata/issue3164"); !os.IsNotExist(err) {
+		t.Fatal(err)
+	}
+}
+
+func TestScanNoDatabaseWrite(t *testing.T) {
+	// When scanning, nothing should be committed to database unless
+	// something actually changed.
+
+	db := db.OpenMemory()
+	m := NewModel(defaultConfig, protocol.LocalDeviceID, "device", "syncthing", "dev", db, nil)
+	m.AddFolder(defaultFolderConfig)
+	m.StartFolder("default")
+	m.ServeBackground()
+
+	// Start with no ignores, and restore the previous state when the test completes
+
+	curIgn, _, err := m.GetIgnores("default")
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer m.SetIgnores("default", curIgn)
+	m.SetIgnores("default", nil)
+
+	// Scan the folder twice. The second scan should be a no-op database wise
+
+	m.ScanFolder("default")
+	c0 := db.Committed()
+
+	m.ScanFolder("default")
+	c1 := db.Committed()
+
+	if c1 != c0 {
+		t.Errorf("scan should not commit data when nothing changed but %d != %d", c1, c0)
+	}
+
+	// Ignore a file we know exists. It'll be updated in the database.
+
+	m.SetIgnores("default", []string{"foo"})
+
+	m.ScanFolder("default")
+	c2 := db.Committed()
+
+	if c2 <= c1 {
+		t.Errorf("scan should commit data when something got ignored but %d <= %d", c2, c1)
+	}
+
+	// Scan again. Nothing should happen.
+
+	m.ScanFolder("default")
+	c3 := db.Committed()
+
+	if c3 != c2 {
+		t.Errorf("scan should not commit data when nothing changed (with ignores) but %d != %d", c3, c2)
+	}
+}
+
+func TestIssue2782(t *testing.T) {
+	// CheckFolderHealth should accept a symlinked folder, when using tilde-expanded path.
+
+	if runtime.GOOS == "windows" {
+		t.Skip("not reliable on Windows")
+		return
+	}
+	home := os.Getenv("HOME")
+	if home == "" {
+		t.Skip("no home")
+	}
+
+	// Create the test env. Needs to be based on $HOME as tilde expansion is
+	// part of the issue. Skip the test if any of this fails, as we are a
+	// bit outside of our stated domain here...
+
+	testName := ".syncthing-test." + srand.String(16)
+	testDir := filepath.Join(home, testName)
+	if err := osutil.RemoveAll(testDir); err != nil {
+		t.Skip(err)
+	}
+	if err := osutil.MkdirAll(testDir+"/syncdir", 0755); err != nil {
+		t.Skip(err)
+	}
+	if err := ioutil.WriteFile(testDir+"/syncdir/file", []byte("hello, world\n"), 0644); err != nil {
+		t.Skip(err)
+	}
+	if err := os.Symlink("syncdir", testDir+"/synclink"); err != nil {
+		t.Skip(err)
+	}
+	defer osutil.RemoveAll(testDir)
+
+	db := db.OpenMemory()
+	m := NewModel(defaultConfig, protocol.LocalDeviceID, "device", "syncthing", "dev", db, nil)
+	m.AddFolder(config.NewFolderConfiguration("default", "~/"+testName+"/synclink/"))
+	m.StartFolder("default")
+	m.ServeBackground()
+	defer m.Stop()
+
+	if err := m.ScanFolder("default"); err != nil {
+		t.Error("scan error:", err)
+	}
+
+	if err := m.CheckFolderHealth("default"); err != nil {
+		t.Error("health check error:", err)
+	}
+}
+
 type fakeAddr struct{}
 
 func (fakeAddr) Network() string {
